#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: doppler_radar_jupiter
# GNU Radio version: v3.10.11.0-1-gee27d6f3

from PyQt5 import Qt
from gnuradio import qtgui
from PyQt5 import QtCore
from gnuradio import analog
from gnuradio import audio
from gnuradio import blocks
from gnuradio import filter
from gnuradio.filter import firdes
from gnuradio import gr
from gnuradio.fft import window
import sys
import signal
from PyQt5 import Qt
from argparse import ArgumentParser
from gnuradio.eng_arg import eng_float, intx
from gnuradio import eng_notation
from gnuradio import iio
import configparser
import sip
import threading



class doppler_radar_jupiter(gr.top_block, Qt.QWidget):

    def __init__(self):
        gr.top_block.__init__(self, "doppler_radar_jupiter", catch_exceptions=True)
        Qt.QWidget.__init__(self)
        self.setWindowTitle("doppler_radar_jupiter")
        qtgui.util.check_set_qss()
        try:
            self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
        except BaseException as exc:
            print(f"Qt GUI: Could not set Icon: {str(exc)}", file=sys.stderr)
        self.top_scroll_layout = Qt.QVBoxLayout()
        self.setLayout(self.top_scroll_layout)
        self.top_scroll = Qt.QScrollArea()
        self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
        self.top_scroll_layout.addWidget(self.top_scroll)
        self.top_scroll.setWidgetResizable(True)
        self.top_widget = Qt.QWidget()
        self.top_scroll.setWidget(self.top_widget)
        self.top_layout = Qt.QVBoxLayout(self.top_widget)
        self.top_grid_layout = Qt.QGridLayout()
        self.top_layout.addLayout(self.top_grid_layout)

        self.settings = Qt.QSettings("gnuradio/flowgraphs", "doppler_radar_jupiter")

        try:
            geometry = self.settings.value("geometry")
            if geometry:
                self.restoreGeometry(geometry)
        except BaseException as exc:
            print(f"Qt GUI: Could not restore geometry: {str(exc)}", file=sys.stderr)
        self.flowgraph_started = threading.Event()

        ##################################################
        # Variables
        ##################################################
        self.tx_lo_freq = tx_lo_freq = int(5674000000)
        self.transmit_sine_freq = transmit_sine_freq = 1500
        self.samp_rate = samp_rate = 1920000
        self.rx_lo_freq = rx_lo_freq = int(5674000000)
        self.lo_offset = lo_offset = int(150000)
        self._jupiter_ip_config = configparser.ConfigParser()
        self._jupiter_ip_config.read('default')
        try: jupiter_ip = self._jupiter_ip_config.get('main', 'key')
        except: jupiter_ip = 'ip:jupiter.local'
        self.jupiter_ip = jupiter_ip
        self.interval_update = interval_update = int(0)
        self.hardwaregain_tx0 = hardwaregain_tx0 = 0
        self.hardwaregain_rx0 = hardwaregain_rx0 = 20
        self.gain_control_mode_rx0 = gain_control_mode_rx0 = 'spi'
        self.decimation_2 = decimation_2 = 1
        self.decimation_1 = decimation_1 = 40
        self.buff_size = buff_size = int(32768)
        self.audio_gain = audio_gain = 50

        ##################################################
        # Blocks
        ##################################################

        self._hardwaregain_tx0_range = qtgui.Range(-41, 0, 1, 0, 200)
        self._hardwaregain_tx0_win = qtgui.RangeWidget(self._hardwaregain_tx0_range, self.set_hardwaregain_tx0, "Tx Attenuation [dBfs]", "counter_slider", float, QtCore.Qt.Horizontal)
        self.top_layout.addWidget(self._hardwaregain_tx0_win)
        self._hardwaregain_rx0_range = qtgui.Range(0, 34, 1, 20, 200)
        self._hardwaregain_rx0_win = qtgui.RangeWidget(self._hardwaregain_rx0_range, self.set_hardwaregain_rx0, "Rx Gain [dBfs]", "counter_slider", float, QtCore.Qt.Horizontal)
        self.top_layout.addWidget(self._hardwaregain_rx0_win)
        self._audio_gain_range = qtgui.Range(0, 60, 1, 50, 200)
        self._audio_gain_win = qtgui.RangeWidget(self._audio_gain_range, self.set_audio_gain, "'audio_gain'", "counter_slider", float, QtCore.Qt.Horizontal)
        self.top_layout.addWidget(self._audio_gain_win)
        self.qtgui_time_sink_x_2 = qtgui.time_sink_c(
            (1024*2*2), #size
            samp_rate/decimation_1/decimation_2, #samp_rate
            'Time Domain BRF', #name
            1, #number of inputs
            None # parent
        )
        self.qtgui_time_sink_x_2.set_update_time(0.10)
        self.qtgui_time_sink_x_2.set_y_axis(-0.01, 0.01)

        self.qtgui_time_sink_x_2.set_y_label('Amplitude', "")

        self.qtgui_time_sink_x_2.enable_tags(True)
        self.qtgui_time_sink_x_2.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, 0, "")
        self.qtgui_time_sink_x_2.enable_autoscale(False)
        self.qtgui_time_sink_x_2.enable_grid(False)
        self.qtgui_time_sink_x_2.enable_axis_labels(True)
        self.qtgui_time_sink_x_2.enable_control_panel(False)
        self.qtgui_time_sink_x_2.enable_stem_plot(False)


        labels = ['Rx BRF Re', 'Rx BRF Im', 'Signal 3', 'Signal 4', 'Signal 5',
            'Signal 6', 'Signal 7', 'Signal 8', 'Signal 9', 'Signal 10']
        widths = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        colors = ['blue', 'red', 'green', 'black', 'cyan',
            'magenta', 'yellow', 'dark red', 'dark green', 'dark blue']
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]
        styles = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        markers = [-1, -1, -1, -1, -1,
            -1, -1, -1, -1, -1]


        for i in range(2):
            if len(labels[i]) == 0:
                if (i % 2 == 0):
                    self.qtgui_time_sink_x_2.set_line_label(i, "Re{{Data {0}}}".format(i/2))
                else:
                    self.qtgui_time_sink_x_2.set_line_label(i, "Im{{Data {0}}}".format(i/2))
            else:
                self.qtgui_time_sink_x_2.set_line_label(i, labels[i])
            self.qtgui_time_sink_x_2.set_line_width(i, widths[i])
            self.qtgui_time_sink_x_2.set_line_color(i, colors[i])
            self.qtgui_time_sink_x_2.set_line_style(i, styles[i])
            self.qtgui_time_sink_x_2.set_line_marker(i, markers[i])
            self.qtgui_time_sink_x_2.set_line_alpha(i, alphas[i])

        self._qtgui_time_sink_x_2_win = sip.wrapinstance(self.qtgui_time_sink_x_2.qwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_time_sink_x_2_win, 2, 0, 1, 1)
        for r in range(2, 3):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(0, 1):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_time_sink_x_1 = qtgui.time_sink_c(
            (1024*2*2), #size
            samp_rate/decimation_1, #samp_rate
            'Time domain Rx BPF', #name
            1, #number of inputs
            None # parent
        )
        self.qtgui_time_sink_x_1.set_update_time(0.10)
        self.qtgui_time_sink_x_1.set_y_axis(-0.2, 0.2)

        self.qtgui_time_sink_x_1.set_y_label('Amplitude', "")

        self.qtgui_time_sink_x_1.enable_tags(True)
        self.qtgui_time_sink_x_1.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, 0, "")
        self.qtgui_time_sink_x_1.enable_autoscale(False)
        self.qtgui_time_sink_x_1.enable_grid(True)
        self.qtgui_time_sink_x_1.enable_axis_labels(True)
        self.qtgui_time_sink_x_1.enable_control_panel(False)
        self.qtgui_time_sink_x_1.enable_stem_plot(False)


        labels = ['Rx BPF Re', 'Rx BPF Im', 'Signal 3', 'Signal 4', 'Signal 5',
            'Signal 6', 'Signal 7', 'Signal 8', 'Signal 9', 'Signal 10']
        widths = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        colors = ['blue', 'red', 'green', 'black', 'cyan',
            'magenta', 'yellow', 'dark red', 'dark green', 'dark blue']
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]
        styles = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        markers = [-1, -1, -1, -1, -1,
            -1, -1, -1, -1, -1]


        for i in range(2):
            if len(labels[i]) == 0:
                if (i % 2 == 0):
                    self.qtgui_time_sink_x_1.set_line_label(i, "Re{{Data {0}}}".format(i/2))
                else:
                    self.qtgui_time_sink_x_1.set_line_label(i, "Im{{Data {0}}}".format(i/2))
            else:
                self.qtgui_time_sink_x_1.set_line_label(i, labels[i])
            self.qtgui_time_sink_x_1.set_line_width(i, widths[i])
            self.qtgui_time_sink_x_1.set_line_color(i, colors[i])
            self.qtgui_time_sink_x_1.set_line_style(i, styles[i])
            self.qtgui_time_sink_x_1.set_line_marker(i, markers[i])
            self.qtgui_time_sink_x_1.set_line_alpha(i, alphas[i])

        self._qtgui_time_sink_x_1_win = sip.wrapinstance(self.qtgui_time_sink_x_1.qwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_time_sink_x_1_win, 1, 0, 1, 1)
        for r in range(1, 2):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(0, 1):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_time_sink_x_0 = qtgui.time_sink_c(
            (1024*2*2), #size
            samp_rate, #samp_rate
            'Time Domain Rx Raw', #name
            1, #number of inputs
            None # parent
        )
        self.qtgui_time_sink_x_0.set_update_time(0.10)
        self.qtgui_time_sink_x_0.set_y_axis(-0.2, 0.2)

        self.qtgui_time_sink_x_0.set_y_label('Amplitude', "")

        self.qtgui_time_sink_x_0.enable_tags(True)
        self.qtgui_time_sink_x_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, 0, "")
        self.qtgui_time_sink_x_0.enable_autoscale(False)
        self.qtgui_time_sink_x_0.enable_grid(False)
        self.qtgui_time_sink_x_0.enable_axis_labels(True)
        self.qtgui_time_sink_x_0.enable_control_panel(False)
        self.qtgui_time_sink_x_0.enable_stem_plot(False)


        labels = ['Rx Raw Re', 'Rx Raw Im', 'Signal 3', 'Signal 4', 'Signal 5',
            'Signal 6', 'Signal 7', 'Signal 8', 'Signal 9', 'Signal 10']
        widths = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        colors = ['blue', 'red', 'green', 'black', 'cyan',
            'magenta', 'yellow', 'dark red', 'dark green', 'dark blue']
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]
        styles = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        markers = [-1, -1, -1, -1, -1,
            -1, -1, -1, -1, -1]


        for i in range(2):
            if len(labels[i]) == 0:
                if (i % 2 == 0):
                    self.qtgui_time_sink_x_0.set_line_label(i, "Re{{Data {0}}}".format(i/2))
                else:
                    self.qtgui_time_sink_x_0.set_line_label(i, "Im{{Data {0}}}".format(i/2))
            else:
                self.qtgui_time_sink_x_0.set_line_label(i, labels[i])
            self.qtgui_time_sink_x_0.set_line_width(i, widths[i])
            self.qtgui_time_sink_x_0.set_line_color(i, colors[i])
            self.qtgui_time_sink_x_0.set_line_style(i, styles[i])
            self.qtgui_time_sink_x_0.set_line_marker(i, markers[i])
            self.qtgui_time_sink_x_0.set_line_alpha(i, alphas[i])

        self._qtgui_time_sink_x_0_win = sip.wrapinstance(self.qtgui_time_sink_x_0.qwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_time_sink_x_0_win, 0, 0, 1, 1)
        for r in range(0, 1):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(0, 1):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_freq_sink_x_1 = qtgui.freq_sink_c(
            16384, #size
            window.WIN_BLACKMAN_hARRIS, #wintype
            0, #fc
            (samp_rate/decimation_1/decimation_2), #bw
            'Frequency Spectrum Rx BRF', #name
            1,
            None # parent
        )
        self.qtgui_freq_sink_x_1.set_update_time(0.10)
        self.qtgui_freq_sink_x_1.set_y_axis((-240), 10)
        self.qtgui_freq_sink_x_1.set_y_label('Relative Gain', 'dB')
        self.qtgui_freq_sink_x_1.set_trigger_mode(qtgui.TRIG_MODE_FREE, 0.0, 0, "")
        self.qtgui_freq_sink_x_1.enable_autoscale(False)
        self.qtgui_freq_sink_x_1.enable_grid(True)
        self.qtgui_freq_sink_x_1.set_fft_average(1.0)
        self.qtgui_freq_sink_x_1.enable_axis_labels(True)
        self.qtgui_freq_sink_x_1.enable_control_panel(False)
        self.qtgui_freq_sink_x_1.set_fft_window_normalized(False)



        labels = ['', '', '', '', '',
            '', '', '', '', '']
        widths = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
            "magenta", "yellow", "dark red", "dark green", "dark blue"]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]

        for i in range(1):
            if len(labels[i]) == 0:
                self.qtgui_freq_sink_x_1.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_freq_sink_x_1.set_line_label(i, labels[i])
            self.qtgui_freq_sink_x_1.set_line_width(i, widths[i])
            self.qtgui_freq_sink_x_1.set_line_color(i, colors[i])
            self.qtgui_freq_sink_x_1.set_line_alpha(i, alphas[i])

        self._qtgui_freq_sink_x_1_win = sip.wrapinstance(self.qtgui_freq_sink_x_1.qwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_freq_sink_x_1_win, 2, 1, 1, 1)
        for r in range(2, 3):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(1, 2):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_freq_sink_x_0_0 = qtgui.freq_sink_c(
            8192, #size
            window.WIN_BLACKMAN_hARRIS, #wintype
            0, #fc
            (samp_rate/decimation_1), #bw
            'Frequency Spectrum Rx BPF', #name
            1,
            None # parent
        )
        self.qtgui_freq_sink_x_0_0.set_update_time(0.10)
        self.qtgui_freq_sink_x_0_0.set_y_axis((-240), 10)
        self.qtgui_freq_sink_x_0_0.set_y_label('Relative Gain', 'dB')
        self.qtgui_freq_sink_x_0_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, 0.0, 0, "")
        self.qtgui_freq_sink_x_0_0.enable_autoscale(False)
        self.qtgui_freq_sink_x_0_0.enable_grid(False)
        self.qtgui_freq_sink_x_0_0.set_fft_average(1.0)
        self.qtgui_freq_sink_x_0_0.enable_axis_labels(True)
        self.qtgui_freq_sink_x_0_0.enable_control_panel(False)
        self.qtgui_freq_sink_x_0_0.set_fft_window_normalized(False)



        labels = ['Rx BPF', '', '', '', '',
            '', '', '', '', '']
        widths = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
            "magenta", "yellow", "dark red", "dark green", "dark blue"]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]

        for i in range(1):
            if len(labels[i]) == 0:
                self.qtgui_freq_sink_x_0_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_freq_sink_x_0_0.set_line_label(i, labels[i])
            self.qtgui_freq_sink_x_0_0.set_line_width(i, widths[i])
            self.qtgui_freq_sink_x_0_0.set_line_color(i, colors[i])
            self.qtgui_freq_sink_x_0_0.set_line_alpha(i, alphas[i])

        self._qtgui_freq_sink_x_0_0_win = sip.wrapinstance(self.qtgui_freq_sink_x_0_0.qwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_freq_sink_x_0_0_win, 1, 1, 1, 1)
        for r in range(1, 2):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(1, 2):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_freq_sink_x_0 = qtgui.freq_sink_c(
            16384, #size
            window.WIN_BLACKMAN_hARRIS, #wintype
            0, #fc
            samp_rate, #bw
            'Frequency Spectrum Rx Raw', #name
            1,
            None # parent
        )
        self.qtgui_freq_sink_x_0.set_update_time(0.10)
        self.qtgui_freq_sink_x_0.set_y_axis((-240), 10)
        self.qtgui_freq_sink_x_0.set_y_label('Relative Gain', 'dB')
        self.qtgui_freq_sink_x_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, 0.0, 0, "")
        self.qtgui_freq_sink_x_0.enable_autoscale(False)
        self.qtgui_freq_sink_x_0.enable_grid(False)
        self.qtgui_freq_sink_x_0.set_fft_average(1.0)
        self.qtgui_freq_sink_x_0.enable_axis_labels(True)
        self.qtgui_freq_sink_x_0.enable_control_panel(False)
        self.qtgui_freq_sink_x_0.set_fft_window_normalized(False)



        labels = ['Rx Raw', '', '', '', '',
            '', '', '', '', '']
        widths = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
            "magenta", "yellow", "dark red", "dark green", "dark blue"]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]

        for i in range(1):
            if len(labels[i]) == 0:
                self.qtgui_freq_sink_x_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_freq_sink_x_0.set_line_label(i, labels[i])
            self.qtgui_freq_sink_x_0.set_line_width(i, widths[i])
            self.qtgui_freq_sink_x_0.set_line_color(i, colors[i])
            self.qtgui_freq_sink_x_0.set_line_alpha(i, alphas[i])

        self._qtgui_freq_sink_x_0_win = sip.wrapinstance(self.qtgui_freq_sink_x_0.qwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_freq_sink_x_0_win, 0, 1, 1, 1)
        for r in range(0, 1):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(1, 2):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.iio_device_source_0 = iio.device_source(jupiter_ip, 'axi-adrv9002-rx-lpc', ['voltage0_i','voltage0_q'], 'adrv9002-phy', [], buff_size, 1 - 1)
        self.iio_device_source_0.set_len_tag_key('packet_len')
        self.iio_device_sink_0_1 = iio.device_sink(jupiter_ip, 'axi-adrv9002-tx-lpc', ['voltage0','voltage1'], 'adrv9002-phy', [], buff_size, 1 - 1, False)
        self.iio_device_sink_0_1.set_len_tag_key('')
        self.iio_attr_updater_0_1_0_1 = iio.attr_updater('gain_control_mode', gain_control_mode_rx0, interval_update)
        self.iio_attr_updater_0_1_0_0_0 = iio.attr_updater('initial_calibrations', 'auto', interval_update)
        self.iio_attr_updater_0_1_0_0 = iio.attr_updater('frequency', str(tx_lo_freq), interval_update)
        self.iio_attr_updater_0_1_0 = iio.attr_updater('frequency', str(rx_lo_freq), interval_update)
        self.iio_attr_updater_0_1 = iio.attr_updater('hardwaregain', str(hardwaregain_tx0), interval_update)
        self.iio_attr_updater_0_0_0 = iio.attr_updater('hardwaregain', str(-40), interval_update)
        self.iio_attr_updater_0_0 = iio.attr_updater('hardwaregain', str(0), interval_update)
        self.iio_attr_updater_0 = iio.attr_updater('hardwaregain', str(hardwaregain_rx0), interval_update)
        self.iio_attr_sink_0_1_0_1 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'voltage0', 0, False)
        self.iio_attr_sink_0_1_0_0_0 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'adrv9002-phy', 1, False)
        self.iio_attr_sink_0_1_0_0 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'altvoltage2', 0, True)
        self.iio_attr_sink_0_1_0 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'altvoltage0', 0, True)
        self.iio_attr_sink_0_1 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'voltage0', 0, True)
        self.iio_attr_sink_0_0_0 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'voltage1', 0, True)
        self.iio_attr_sink_0_0 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'voltage1', 0, False)
        self.iio_attr_sink_0 = iio.attr_sink(jupiter_ip, 'adrv9002-phy', 'voltage0', 0, False)
        self.blocks_short_to_float_0_1 = blocks.short_to_float(1, 32767)
        self.blocks_short_to_float_0 = blocks.short_to_float(1, 32767)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_cc(audio_gain)
        self.blocks_float_to_short_0_5_0 = blocks.float_to_short(1, 32767)
        self.blocks_float_to_short_0_0_4_0 = blocks.float_to_short(1, 32767)
        self.blocks_float_to_complex_0 = blocks.float_to_complex(1)
        self.blocks_complex_to_real_0_4_0 = blocks.complex_to_real(1)
        self.blocks_complex_to_real_0 = blocks.complex_to_real(1)
        self.blocks_complex_to_imag_0_4_0 = blocks.complex_to_imag(1)
        self.band_reject_filter_0 = filter.fir_filter_ccf(
            1,
            firdes.band_reject(
                1,
                (samp_rate/decimation_1/decimation_2),
                (transmit_sine_freq - 20),
                (transmit_sine_freq + 20),
                30,
                window.WIN_HAMMING,
                6.76))
        self.band_pass_filter_0 = filter.fir_filter_ccf(
            decimation_1,
            firdes.band_pass(
                1,
                samp_rate,
                (transmit_sine_freq - 300),
                (transmit_sine_freq + 300),
                1500,
                window.WIN_HAMMING,
                6.76))
        self.audio_sink_0 = audio.sink(48000, 'default', True)
        self.analog_sig_source_x_0 = analog.sig_source_c(samp_rate, analog.GR_COS_WAVE, transmit_sine_freq, 1, 0, 0)


        ##################################################
        # Connections
        ##################################################
        self.msg_connect((self.iio_attr_updater_0, 'out'), (self.iio_attr_sink_0, 'attr'))
        self.msg_connect((self.iio_attr_updater_0_0, 'out'), (self.iio_attr_sink_0_0, 'attr'))
        self.msg_connect((self.iio_attr_updater_0_0_0, 'out'), (self.iio_attr_sink_0_0_0, 'attr'))
        self.msg_connect((self.iio_attr_updater_0_1, 'out'), (self.iio_attr_sink_0_1, 'attr'))
        self.msg_connect((self.iio_attr_updater_0_1_0, 'out'), (self.iio_attr_sink_0_1_0, 'attr'))
        self.msg_connect((self.iio_attr_updater_0_1_0_0, 'out'), (self.iio_attr_sink_0_1_0_0, 'attr'))
        self.msg_connect((self.iio_attr_updater_0_1_0_0_0, 'out'), (self.iio_attr_sink_0_1_0_0_0, 'attr'))
        self.msg_connect((self.iio_attr_updater_0_1_0_1, 'out'), (self.iio_attr_sink_0_1_0_1, 'attr'))
        self.connect((self.analog_sig_source_x_0, 0), (self.blocks_complex_to_imag_0_4_0, 0))
        self.connect((self.analog_sig_source_x_0, 0), (self.blocks_complex_to_real_0_4_0, 0))
        self.connect((self.band_pass_filter_0, 0), (self.band_reject_filter_0, 0))
        self.connect((self.band_pass_filter_0, 0), (self.qtgui_freq_sink_x_0_0, 0))
        self.connect((self.band_pass_filter_0, 0), (self.qtgui_time_sink_x_1, 0))
        self.connect((self.band_reject_filter_0, 0), (self.blocks_multiply_const_vxx_0, 0))
        self.connect((self.band_reject_filter_0, 0), (self.qtgui_freq_sink_x_1, 0))
        self.connect((self.band_reject_filter_0, 0), (self.qtgui_time_sink_x_2, 0))
        self.connect((self.blocks_complex_to_imag_0_4_0, 0), (self.blocks_float_to_short_0_0_4_0, 0))
        self.connect((self.blocks_complex_to_real_0, 0), (self.audio_sink_0, 0))
        self.connect((self.blocks_complex_to_real_0_4_0, 0), (self.blocks_float_to_short_0_5_0, 0))
        self.connect((self.blocks_float_to_complex_0, 0), (self.band_pass_filter_0, 0))
        self.connect((self.blocks_float_to_complex_0, 0), (self.qtgui_freq_sink_x_0, 0))
        self.connect((self.blocks_float_to_complex_0, 0), (self.qtgui_time_sink_x_0, 0))
        self.connect((self.blocks_float_to_short_0_0_4_0, 0), (self.iio_device_sink_0_1, 1))
        self.connect((self.blocks_float_to_short_0_5_0, 0), (self.iio_device_sink_0_1, 0))
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.blocks_complex_to_real_0, 0))
        self.connect((self.blocks_short_to_float_0, 0), (self.blocks_float_to_complex_0, 0))
        self.connect((self.blocks_short_to_float_0_1, 0), (self.blocks_float_to_complex_0, 1))
        self.connect((self.iio_device_source_0, 0), (self.blocks_short_to_float_0, 0))
        self.connect((self.iio_device_source_0, 1), (self.blocks_short_to_float_0_1, 0))


    def closeEvent(self, event):
        self.settings = Qt.QSettings("gnuradio/flowgraphs", "doppler_radar_jupiter")
        self.settings.setValue("geometry", self.saveGeometry())
        self.stop()
        self.wait()

        event.accept()

    def get_tx_lo_freq(self):
        return self.tx_lo_freq

    def set_tx_lo_freq(self, tx_lo_freq):
        self.tx_lo_freq = tx_lo_freq
        self.iio_attr_updater_0_1_0_0.set_value(str(self.tx_lo_freq))

    def get_transmit_sine_freq(self):
        return self.transmit_sine_freq

    def set_transmit_sine_freq(self, transmit_sine_freq):
        self.transmit_sine_freq = transmit_sine_freq
        self.analog_sig_source_x_0.set_frequency(self.transmit_sine_freq)
        self.band_pass_filter_0.set_taps(firdes.band_pass(1, self.samp_rate, (self.transmit_sine_freq - 300), (self.transmit_sine_freq + 300), 1500, window.WIN_HAMMING, 6.76))
        self.band_reject_filter_0.set_taps(firdes.band_reject(1, (self.samp_rate/self.decimation_1/self.decimation_2), (self.transmit_sine_freq - 20), (self.transmit_sine_freq + 20), 30, window.WIN_HAMMING, 6.76))

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.analog_sig_source_x_0.set_sampling_freq(self.samp_rate)
        self.band_pass_filter_0.set_taps(firdes.band_pass(1, self.samp_rate, (self.transmit_sine_freq - 300), (self.transmit_sine_freq + 300), 1500, window.WIN_HAMMING, 6.76))
        self.band_reject_filter_0.set_taps(firdes.band_reject(1, (self.samp_rate/self.decimation_1/self.decimation_2), (self.transmit_sine_freq - 20), (self.transmit_sine_freq + 20), 30, window.WIN_HAMMING, 6.76))
        self.qtgui_freq_sink_x_0.set_frequency_range(0, self.samp_rate)
        self.qtgui_freq_sink_x_0_0.set_frequency_range(0, (self.samp_rate/self.decimation_1))
        self.qtgui_freq_sink_x_1.set_frequency_range(0, (self.samp_rate/self.decimation_1/self.decimation_2))
        self.qtgui_time_sink_x_0.set_samp_rate(self.samp_rate)
        self.qtgui_time_sink_x_1.set_samp_rate(self.samp_rate/self.decimation_1)
        self.qtgui_time_sink_x_2.set_samp_rate(self.samp_rate/self.decimation_1/self.decimation_2)

    def get_rx_lo_freq(self):
        return self.rx_lo_freq

    def set_rx_lo_freq(self, rx_lo_freq):
        self.rx_lo_freq = rx_lo_freq
        self.iio_attr_updater_0_1_0.set_value(str(self.rx_lo_freq))

    def get_lo_offset(self):
        return self.lo_offset

    def set_lo_offset(self, lo_offset):
        self.lo_offset = lo_offset

    def get_jupiter_ip(self):
        return self.jupiter_ip

    def set_jupiter_ip(self, jupiter_ip):
        self.jupiter_ip = jupiter_ip

    def get_interval_update(self):
        return self.interval_update

    def set_interval_update(self, interval_update):
        self.interval_update = interval_update

    def get_hardwaregain_tx0(self):
        return self.hardwaregain_tx0

    def set_hardwaregain_tx0(self, hardwaregain_tx0):
        self.hardwaregain_tx0 = hardwaregain_tx0
        self.iio_attr_updater_0_1.set_value(str(self.hardwaregain_tx0))

    def get_hardwaregain_rx0(self):
        return self.hardwaregain_rx0

    def set_hardwaregain_rx0(self, hardwaregain_rx0):
        self.hardwaregain_rx0 = hardwaregain_rx0
        self.iio_attr_updater_0.set_value(str(self.hardwaregain_rx0))

    def get_gain_control_mode_rx0(self):
        return self.gain_control_mode_rx0

    def set_gain_control_mode_rx0(self, gain_control_mode_rx0):
        self.gain_control_mode_rx0 = gain_control_mode_rx0
        self.iio_attr_updater_0_1_0_1.set_value(self.gain_control_mode_rx0)

    def get_decimation_2(self):
        return self.decimation_2

    def set_decimation_2(self, decimation_2):
        self.decimation_2 = decimation_2
        self.band_reject_filter_0.set_taps(firdes.band_reject(1, (self.samp_rate/self.decimation_1/self.decimation_2), (self.transmit_sine_freq - 20), (self.transmit_sine_freq + 20), 30, window.WIN_HAMMING, 6.76))
        self.qtgui_freq_sink_x_1.set_frequency_range(0, (self.samp_rate/self.decimation_1/self.decimation_2))
        self.qtgui_time_sink_x_2.set_samp_rate(self.samp_rate/self.decimation_1/self.decimation_2)

    def get_decimation_1(self):
        return self.decimation_1

    def set_decimation_1(self, decimation_1):
        self.decimation_1 = decimation_1
        self.band_reject_filter_0.set_taps(firdes.band_reject(1, (self.samp_rate/self.decimation_1/self.decimation_2), (self.transmit_sine_freq - 20), (self.transmit_sine_freq + 20), 30, window.WIN_HAMMING, 6.76))
        self.qtgui_freq_sink_x_0_0.set_frequency_range(0, (self.samp_rate/self.decimation_1))
        self.qtgui_freq_sink_x_1.set_frequency_range(0, (self.samp_rate/self.decimation_1/self.decimation_2))
        self.qtgui_time_sink_x_1.set_samp_rate(self.samp_rate/self.decimation_1)
        self.qtgui_time_sink_x_2.set_samp_rate(self.samp_rate/self.decimation_1/self.decimation_2)

    def get_buff_size(self):
        return self.buff_size

    def set_buff_size(self, buff_size):
        self.buff_size = buff_size

    def get_audio_gain(self):
        return self.audio_gain

    def set_audio_gain(self, audio_gain):
        self.audio_gain = audio_gain
        self.blocks_multiply_const_vxx_0.set_k(self.audio_gain)




def main(top_block_cls=doppler_radar_jupiter, options=None):

    qapp = Qt.QApplication(sys.argv)

    tb = top_block_cls()

    tb.start()
    tb.flowgraph_started.set()

    tb.show()

    def sig_handler(sig=None, frame=None):
        tb.stop()
        tb.wait()

        Qt.QApplication.quit()

    signal.signal(signal.SIGINT, sig_handler)
    signal.signal(signal.SIGTERM, sig_handler)

    timer = Qt.QTimer()
    timer.start(500)
    timer.timeout.connect(lambda: None)

    qapp.exec_()

if __name__ == '__main__':
    main()
